"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformSymbolConstructorCall = void 0;
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const lualib_1 = require("../utils/lualib");
const call_1 = require("../visitors/call");
function transformSymbolConstructorCall(context, expression) {
    const method = expression.expression;
    const signature = context.checker.getResolvedSignature(expression);
    const parameters = (0, call_1.transformArguments)(context, expression.arguments, signature);
    const methodName = method.name.text;
    switch (methodName) {
        case "for":
        case "keyFor":
            (0, lualib_1.importLuaLibFeature)(context, lualib_1.LuaLibFeature.SymbolRegistry);
            const upperMethodName = methodName[0].toUpperCase() + methodName.slice(1);
            const functionIdentifier = lua.createIdentifier(`__TS__SymbolRegistry${upperMethodName}`);
            return lua.createCallExpression(functionIdentifier, parameters, expression);
        default:
            context.diagnostics.push((0, diagnostics_1.unsupportedProperty)(method.name, "Symbol", methodName));
    }
}
exports.transformSymbolConstructorCall = transformSymbolConstructorCall;
//# sourceMappingURL=symbol.js.map